Package: nc
Authors@R: c(
    person("Toby", "Hocking",
     email="toby.hocking@r-project.org",
     role=c("aut", "cre")))
Version: 2026.4.20
License: GPL-3
Title: Named Capture to Data Tables
Description: User-friendly functions for extracting a data
 table (row for each match, column for each group)
 from non-tabular text data using regular expressions,
 and for melting columns that match a regular expression.
 Patterns are defined using a readable syntax
 that makes it easy to build complex patterns
 in terms of simpler, re-usable sub-patterns.
 Named R arguments are translated to column names
 in the output; capture groups without names are used
 internally in order to provide a standard interface
 to three regular expression 'C' libraries
 ('PCRE', 'RE2', 'ICU').
 Output can also include numeric columns via
 user-specified type conversion functions.
Depends: R (>= 2.14)
Imports: data.table (>= 1.15.0)
Suggests: testthat, re2, stringi, ggplot2, tidyr (>= 1.0.0), cdata,
        reshape2, knitr, markdown, rmarkdown, R.utils, covr, arrow
VignetteBuilder: knitr
URL: https://github.com/tdhock/nc
BugReports: https://github.com/tdhock/nc/issues
NeedsCompilation: no
Packaged: 2026-04-20 19:35:21 UTC; hoct2726
Author: Toby Hocking [aut, cre]
Maintainer: Toby Hocking <toby.hocking@r-project.org>
Repository: CRAN
Date/Publication: 2026-04-21 05:10:12 UTC
Built: R 4.6.0; ; 2026-04-25 20:43:37 UTC; unix
