export(alevels)
export(before_match)
export(altlist)
export(alternatives_with_shared_groups)
export(measure)
export(quantifier)
export(alternatives)
export(field)
export(group)
export(var_args_list)
export(capture_all_str)
export(capture_melt_multiple)
export(capture_melt_single)
export(capture_first_vec)
export(capture_first_glob)
export(capture_first_df)
export(capture_longer_spec)
import(data.table)
importFrom("stats","na.omit")
importFrom("utils","type.convert")
