For now, read the header comments inside each of these for documentation.
The programs are simple enough that they don't really need a Makefile any more
complex than the one given; ymmv.  Data and xor may also be useful on DOS,
which is why there are hooks for it in the code.

data.c		a primitive atob / btoa byte generator
*.d		example input to "data -g"
rservice.c	a utility for scripting up rsh/rexec attacks
xor.c		generic xor handler
