\name{stop_for_subject}
\alias{stop_for_subject}
\title{stop for subject}
\description{Error if \code{subject} incorrect type.}
\usage{stop_for_subject(subject)}
\arguments{
  \item{subject}{character vector.}
}



\author{Toby Hocking <toby.hocking@r-project.org> [aut, cre]}





