\name{stop_for_capture_same_as_id}
\alias{stop_for_capture_same_as_id}
\title{stop for capture same as id}
\description{Error if capture names same as \code{id.vars}.}
\usage{stop_for_capture_same_as_id(capture.vars, 
    id.vars)}
\arguments{
  \item{capture.vars}{character vector of capture column names.}
  \item{id.vars}{character vector of id column names.}
}



\author{Toby Hocking <toby.hocking@r-project.org> [aut, cre]}





